'***************************************  Axes Setup Program  ***************************************
' File:			AX_SETUP.PRG
' Purpose:		Axes setup example
' Version:		V1.0.0.0
' History:		12:30:38 28/09/2017 - V1.0.0.0 Created
' Author:		AUTOMATICALLY GENERATED by AX_SETUP_Gen.py
' Description:	This script configures each axis' parameters according to the user's specifications in
'				the softMC Configurator.
'
' Global Variables Used: num_Of_Motion_Drives as long  - Number of ECat Motion Drives found by EC_SETUP.PRG
'						Axes[64] - An array that holds all the axes in the system. Defined in CONFIG.PRG
'********************************************************************************************************

' module global variables
common shared AX_SETUP_PRG_error_code as long
common shared AX_SETUP_PRG_error_str as string

common shared ERR_AX_SETUP as error "ERROR SETTING UP AXES. CHECK AX_SETUP_PRG_error_code / AX_SETUP_PRG_error_STR" 20765

common shared dHomingFactorFast as double = 0.5
common shared dHomingFactorslow as double = 0.25
common shared dHomingFactorAcc as double = 0.1
common shared dSec2msec as double = 1000  '  this is used to convert from typical user time-units [msec] to typical drive time-units [sec]
common shared dHomingVelocity as double
common shared dHomingAcceleration as double
common shared dVelocityLimit as double
common shared dDrivePeMax as double
common shared mm_to_inch as double = (1/25.4)
common shared pfac_of_drive as double = 1000000
common shared ax_setup_pitch as double = 1

Program
	call Setup_Axis_1

	call Setup_Axis_2

	call Setup_Axis_3

	print "Axes Setup Finished Successfully !"

AX_SETUP_PRG_DONE:

End Program


Sub Setup_Axis_1

	dim retStr as string

	if Execute_VES("with A1", 1) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("AXISTYPE = 0", 1103) <> 0 then
		throw ERR_AX_SETUP
	end if

	try
		ax_setup_pitch = EC_sdo_read(1, 0x207d,0)
	catch 20567
		ax_setup_pitch = 1
	end try

	pfac_of_drive = (EC_SDO_READ(1,0x6092,1)/EC_SDO_READ(1,0x6092,2))*(EC_SDO_READ(1,0x6091,1)/EC_SDO_READ(1,0x6091,2))/ax_setup_pitch/1
	if Execute_VES("PFAC= pfac_of_drive ", 1109) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("VFAC = PFAC/1000", 1109) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("AFAC = PFAC/(1000*1000)", 1109) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("JFAC = PFAC/(1000*1000*1000)", 1109) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("Simulated = 0", 1116) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionErrorDelay = 4.11", 1117) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("MotionBusVelocityScale = 0", 1118) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("MotionBusVelocityBase = 1000", 1119) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("CountMin = -2147483648", 1120) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("CountMax = 2147483647", 1121) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("CountRollOver = 0", 1122) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionExternalFactor = 33333.3", 1123) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("Direction = 1", 1124) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("DirectionExternal = 1", 1125) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("Displacement = 0", 1126) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("VelocityExternalFactor = 1", 1127) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TorqueFactor = 1", 1128) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionErrorSettle = 1.79769e+308", 1129) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TimeSettleMax = -1", 1130) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TimeSettle = 0", 1131) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("Absolute = 0", 1132) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("VelocityCruise = 117.5", 1133) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("Acceleration = 58", 1134) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("Deceleration = 58", 1135) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("DecStop = 1000", 1136) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("Jerk = 15*ACC", 1137) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("Wrap = 0", 1138) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("ProfilerType = -1", 1139) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("SmoothFactor = -1", 1140) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("StartType = 3", 1141) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("StopType = 1", 1142) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("ProceedType = 3", 1143) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("DisableTimeout = 22000", 1144) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionMax = 1.79769e+308", 1145) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionMin = -1.79769e+308", 1146) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionMaxEnable = 0", 1147) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionMinEnable = 0", 1148) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionErrorMax = 10", 1149) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionRollOver = 360", 1150) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionRollOverEnable = 0", 1151) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionRollOverMin = -180", 1152) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionExtRollOver = 360", 1153) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionExtRollOverEnable = 0", 1154) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionExtRollOverMin = -180", 1155) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("VelocityMax = 1000", 1156) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("VelocityOverSpeed = 1.2*VMAX", 1157) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("VelocitySafetyLimit = 10*VMAX", 1158) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("AccelerationMax = 99999", 1159) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("DecelerationMax = 99999", 1160) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("JerkMax = 1.5*DecelerationMax", 1161) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TorqueErrorMax = 100000", 1162) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TorqueMax = 10000", 1163) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TorqueChangeRatioMax = 1", 1164) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("VelocityOverride = 100", 1165) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("VelocityRate = 100", 1166) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("AccelerationRate = 100", 1167) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("DecelerationRate = 100", 1168) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("JerkRate = 100", 1169) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("JerkAccelerationPercentage = 100", 1170) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("JerkDecelerationPercentage = 100", 1171) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("BlendingFactor = 100", 1172) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("BlendingFactorPrevious = -1", 1173) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("BlendingMethod = 1", 1174) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("BlendingStartCondition = 0", 1175) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("CP = 0", 1176) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("CPPrev = -1", 1177) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TorqueErrorDisableType = 0", 1178) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TorqueErrorEnable = 0", 1179) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TorqueErrorFilter = 1.0", 1180) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TorqueErrorStopType = 3", 1181) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TorqueChangeRatio = 1.0", 1182) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("HoldType = 0", 1183) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("DynamicModel = 0", 1184) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("ModelOffset = 0", 1185) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("CoulombFriction = 0", 1186) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("ViscousFriction = 0", 1187) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PayloadInertia = 0", 1188) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PayloadMass = 0", 1189) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("AbsEnable = 1", 1190) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("VelocityLimitTorqueMax = 1000", 1191) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TorqueLimitVelocityMax = 10000", 1192) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TimeOfShortMotions = 0", 1193) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TimeAcceleration = -1", 1194) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TimeDeceleration = -1", 1195) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TimeAccMin = 0", 1196) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TimeDecMin = 0", 1197) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionBacklash = 0", 1198) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionBacklashAlpha = 0.9", 1199) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("FastDataEnable = 0", 1200) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("mastersource = NONE", 1201) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("slave = OFF", 1202) <> 0 then
		throw ERR_AX_SETUP
	end if

	attach A1
		call EC_SET_MOTION_OPMODE(A1, 0)
	detach A1

	retStr = VesExecute("end with")

	print "Done setting motion and homing parameters to axis A1"

End Sub


Sub Setup_Axis_2

	dim retStr as string

	if Execute_VES("with A2", 1) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("AXISTYPE = 0", 1206) <> 0 then
		throw ERR_AX_SETUP
	end if

	try
		ax_setup_pitch = EC_sdo_read(2, 0x207d,0)
	catch 20567
		ax_setup_pitch = 1
	end try

	pfac_of_drive = (EC_SDO_READ(2,0x6092,1)/EC_SDO_READ(2,0x6092,2))*(EC_SDO_READ(2,0x6091,1)/EC_SDO_READ(2,0x6091,2))/ax_setup_pitch/1
	if Execute_VES("PFAC= pfac_of_drive ", 1212) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("VFAC = PFAC/1000", 1212) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("AFAC = PFAC/(1000*1000)", 1212) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("JFAC = PFAC/(1000*1000*1000)", 1212) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("Simulated = 0", 1219) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionErrorDelay = 4.11", 1220) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("MotionBusVelocityScale = 0", 1221) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("MotionBusVelocityBase = 1000", 1222) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("CountMin = -2147483648", 1223) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("CountMax = 2147483647", 1224) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("CountRollOver = 0", 1225) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionExternalFactor = 27397.3", 1226) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("Direction = 1", 1227) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("DirectionExternal = 1", 1228) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("Displacement = 0", 1229) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("VelocityExternalFactor = 1", 1230) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TorqueFactor = 1", 1231) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionErrorSettle = 1.79769e+308", 1232) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TimeSettleMax = -1", 1233) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TimeSettle = 0", 1234) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("Absolute = 0", 1235) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("VelocityCruise = 70", 1236) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("Acceleration = 35", 1237) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("Deceleration = 35", 1238) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("DecStop = 1000", 1239) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("Jerk = 15*ACC", 1240) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("Wrap = 0", 1241) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("ProfilerType = -1", 1242) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("SmoothFactor = -1", 1243) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("StartType = 3", 1244) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("StopType = 1", 1245) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("ProceedType = 3", 1246) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("DisableTimeout = 22000", 1247) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionMax = 1.79769e+308", 1248) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionMin = -1.79769e+308", 1249) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionMaxEnable = 0", 1250) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionMinEnable = 0", 1251) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionErrorMax = 10", 1252) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionRollOver = 360", 1253) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionRollOverEnable = 0", 1254) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionRollOverMin = -180", 1255) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionExtRollOver = 360", 1256) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionExtRollOverEnable = 0", 1257) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionExtRollOverMin = -180", 1258) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("VelocityMax = 1000", 1259) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("VelocityOverSpeed = 1.2*VMAX", 1260) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("VelocitySafetyLimit = 10*VMAX", 1261) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("AccelerationMax = 99999", 1262) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("DecelerationMax = 99999", 1263) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("JerkMax = 1.5*DecelerationMax", 1264) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TorqueErrorMax = 100000", 1265) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TorqueMax = 10000", 1266) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TorqueChangeRatioMax = 1", 1267) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("VelocityOverride = 100", 1268) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("VelocityRate = 100", 1269) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("AccelerationRate = 100", 1270) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("DecelerationRate = 100", 1271) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("JerkRate = 100", 1272) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("JerkAccelerationPercentage = 100", 1273) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("JerkDecelerationPercentage = 100", 1274) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("BlendingFactor = 100", 1275) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("BlendingFactorPrevious = -1", 1276) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("BlendingMethod = 1", 1277) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("BlendingStartCondition = 0", 1278) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("CP = 0", 1279) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("CPPrev = -1", 1280) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TorqueErrorDisableType = 0", 1281) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TorqueErrorEnable = 0", 1282) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TorqueErrorFilter = 1.0", 1283) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TorqueErrorStopType = 3", 1284) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TorqueChangeRatio = 1.0", 1285) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("HoldType = 0", 1286) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("DynamicModel = 0", 1287) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("ModelOffset = 0", 1288) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("CoulombFriction = 0", 1289) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("ViscousFriction = 0", 1290) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PayloadInertia = 0", 1291) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PayloadMass = 0", 1292) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("AbsEnable = 1", 1293) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("VelocityLimitTorqueMax = 1000", 1294) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TorqueLimitVelocityMax = 10000", 1295) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TimeOfShortMotions = 0", 1296) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TimeAcceleration = -1", 1297) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TimeDeceleration = -1", 1298) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TimeAccMin = 0", 1299) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TimeDecMin = 0", 1300) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionBacklash = 0", 1301) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionBacklashAlpha = 0.9", 1302) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("FastDataEnable = 0", 1303) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("mastersource = NONE", 1304) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("slave = OFF", 1305) <> 0 then
		throw ERR_AX_SETUP
	end if

	attach A2
		call EC_SET_MOTION_OPMODE(A2, 0)
	detach A2

	retStr = VesExecute("end with")

	print "Done setting motion and homing parameters to axis A2"

End Sub


Sub Setup_Axis_3

	dim retStr as string

	if Execute_VES("with A3", 1) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("AXISTYPE = 0", 1309) <> 0 then
		throw ERR_AX_SETUP
	end if

	try
		ax_setup_pitch = EC_sdo_read(3, 0x207d,0)
	catch 20567
		ax_setup_pitch = 1
	end try

	pfac_of_drive = (EC_SDO_READ(3,0x6092,1)/EC_SDO_READ(3,0x6092,2))*(EC_SDO_READ(3,0x6091,1)/EC_SDO_READ(3,0x6091,2))/ax_setup_pitch/1
	if Execute_VES("PFAC= pfac_of_drive ", 1315) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("VFAC = PFAC/1000", 1315) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("AFAC = PFAC/(1000*1000)", 1315) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("JFAC = PFAC/(1000*1000*1000)", 1315) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("Simulated = 0", 1322) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionErrorDelay = 4.11", 1323) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("MotionBusVelocityScale = 0", 1324) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("MotionBusVelocityBase = 1000", 1325) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("CountMin = -2147483648", 1326) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("CountMax = 2147483647", 1327) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("CountRollOver = 0", 1328) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionExternalFactor = 27397.3", 1329) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("Direction = 1", 1330) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("DirectionExternal = 1", 1331) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("Displacement = 0", 1332) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("VelocityExternalFactor = 1", 1333) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TorqueFactor = 1", 1334) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionErrorSettle = 1.79769e+308", 1335) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TimeSettleMax = -1", 1336) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TimeSettle = 0", 1337) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("Absolute = 0", 1338) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("VelocityCruise = 12.5", 1339) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("Acceleration = 6.25", 1340) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("Deceleration = 6.25", 1341) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("DecStop = 1000", 1342) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("Jerk = 15*ACC", 1343) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("Wrap = 0", 1344) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("ProfilerType = -1", 1345) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("SmoothFactor = -1", 1346) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("StartType = 3", 1347) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("StopType = 1", 1348) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("ProceedType = 3", 1349) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("DisableTimeout = 22000", 1350) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionMax = 1.79769e+308", 1351) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionMin = -1.79769e+308", 1352) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionMaxEnable = 0", 1353) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionMinEnable = 0", 1354) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionErrorMax = 10", 1355) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionRollOver = 360", 1356) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionRollOverEnable = 0", 1357) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionRollOverMin = -180", 1358) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionExtRollOver = 360", 1359) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionExtRollOverEnable = 0", 1360) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionExtRollOverMin = -180", 1361) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("VelocityMax = 1000", 1362) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("VelocityOverSpeed = 1.2*VMAX", 1363) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("VelocitySafetyLimit = 10*VMAX", 1364) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("AccelerationMax = 99999", 1365) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("DecelerationMax = 99999", 1366) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("JerkMax = 1.5*DecelerationMax", 1367) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TorqueErrorMax = 100000", 1368) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TorqueMax = 10000", 1369) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TorqueChangeRatioMax = 1", 1370) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("VelocityOverride = 100", 1371) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("VelocityRate = 100", 1372) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("AccelerationRate = 100", 1373) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("DecelerationRate = 100", 1374) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("JerkRate = 100", 1375) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("JerkAccelerationPercentage = 100", 1376) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("JerkDecelerationPercentage = 100", 1377) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("BlendingFactor = 100", 1378) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("BlendingFactorPrevious = -1", 1379) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("BlendingMethod = 1", 1380) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("BlendingStartCondition = 0", 1381) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("CP = 0", 1382) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("CPPrev = -1", 1383) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TorqueErrorDisableType = 0", 1384) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TorqueErrorEnable = 0", 1385) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TorqueErrorFilter = 1.0", 1386) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TorqueErrorStopType = 3", 1387) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TorqueChangeRatio = 1.0", 1388) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("HoldType = 0", 1389) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("DynamicModel = 0", 1390) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("ModelOffset = 0", 1391) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("CoulombFriction = 0", 1392) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("ViscousFriction = 0", 1393) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PayloadInertia = 0", 1394) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PayloadMass = 0", 1395) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("AbsEnable = 1", 1396) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("VelocityLimitTorqueMax = 1000", 1397) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TorqueLimitVelocityMax = 10000", 1398) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TimeOfShortMotions = 0", 1399) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TimeAcceleration = -1", 1400) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TimeDeceleration = -1", 1401) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TimeAccMin = 0", 1402) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("TimeDecMin = 0", 1403) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionBacklash = 0", 1404) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("PositionBacklashAlpha = 0.9", 1405) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("FastDataEnable = 0", 1406) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("mastersource = NONE", 1407) <> 0 then
		throw ERR_AX_SETUP
	end if

	if Execute_VES("slave = OFF", 1408) <> 0 then
		throw ERR_AX_SETUP
	end if

	attach A3
		call EC_SET_MOTION_OPMODE(A3, 0)
	detach A3

	retStr = VesExecute("end with")

	print "Done setting motion and homing parameters to axis A3"

End Sub


function Execute_VES(byval command as string, byval err_code as long) as long

	dim retStr as string

	Execute_VES = 0
	AX_SETUP_PRG_error_code = 0
	retStr = VesExecute(command)

	if INSTR(retStr, "E:Error") <> 0  then

		AX_SETUP_PRG_error_code = err_code
		AX_SETUP_PRG_error_str = retStr
		Execute_VES = -1

	end if

end function

